const location = exports.location = {
	DOM_KEY_LOCATION_STANDARD: 0,
	DOM_KEY_LOCATION_LEFT: 1,
	DOM_KEY_LOCATION_RIGHT: 2,
	DOM_KEY_LOCATION_NUMPAD: 3

};
const modifierMap = {
	AltLeft: { key: 'Alt', keycode: 18, location: location.DOM_KEY_LOCATION_LEFT, modifier: 'alt'},
	AltRight: { key: 'Alt', keycode: 18, location: location.DOM_KEY_LOCATION_RIGHT, modifier: 'alt'},
	ControlLeft: { key: 'Control', keycode: 17, location: location.DOM_KEY_LOCATION_LEFT, modifier: 'ctrl'},
	ControlRight: { key: 'Control', keycode: 17, location: location.DOM_KEY_LOCATION_RIGHT, modifier: 'ctrl'},
	MetaLeft: { key: 'Meta', keycode: 91, location: location.DOM_KEY_LOCATION_LEFT, modifier: 'meta'},
	MetaRight: { key: 'Meta', keycode: 93, location: location.DOM_KEY_LOCATION_RIGHT, modifier: 'meta'},
	ShiftLeft: { key: 'Shift', keycode: 16, location: location.DOM_KEY_LOCATION_LEFT, modifier: 'shift'},
	ShiftRight: { key: 'Shift', keycode: 16, location: location.DOM_KEY_LOCATION_RIGHT, modifier: 'shift'},
};

const lockMap = {
	CapsLock: { key: 'CapsLock', keycode: 20, location: location.DOM_KEY_LOCATION_STANDARD },
	NumLock: { key: 'Clear', keycode: 12, location: location.DOM_KEY_LOCATION_NUMPAD },
	ScrollLock: { key: 'ScrollLock', keycode: 12, location: location.DOM_KEY_LOCATION_STANDARD }, //?
};
// Mapping table for Keyboard
const keyboardMap = {
	// 涉及到状态的
	ContextMenu: { key: 'ContextMenu', keycode: 93, press: {keycode: 16}, location: location.DOM_KEY_LOCATION_STANDARD },

	// 不受状态影响的 + Fn?
	Enter: { key: 'Enter', keycode: 13, press: {keycode: 13}, location: location.DOM_KEY_LOCATION_STANDARD },
	Space: { key: ' ', keycode: 32, press: {keycode: 32}, location: location.DOM_KEY_LOCATION_STANDARD },
	Tab: { key: 'Tab', keycode: 9, location: location.DOM_KEY_LOCATION_STANDARD },
	Backspace: { key: 'Backspace', keycode: 8, location: location.DOM_KEY_LOCATION_STANDARD },
	Escape: { key: 'Escape', keycode: 27, location: location.DOM_KEY_LOCATION_STANDARD },
	ArrowLeft: { key: 'ArrowLeft', keycode: 37, location: location.DOM_KEY_LOCATION_STANDARD },
	ArrowUp: { key: 'ArrowUp', keycode: 38, location: location.DOM_KEY_LOCATION_STANDARD },
	ArrowRight: { key: 'ArrowRight', keycode: 39, location: location.DOM_KEY_LOCATION_STANDARD },
	ArrowDown: { key: 'ArrowDown', keycode: 40, location: location.DOM_KEY_LOCATION_STANDARD },
	PageUp: { key: 'PageUp', keycode: 33, location: location.DOM_KEY_LOCATION_STANDARD },
	PageDown: { key: 'PageDown', keycode: 34, location: location.DOM_KEY_LOCATION_STANDARD },
	End: { key: 'End', keycode: 35, location: location.DOM_KEY_LOCATION_STANDARD },
	Home: { key: 'Home', keycode: 36, location: location.DOM_KEY_LOCATION_STANDARD },
	Insert: { key: 'Insert', keycode: 45, location: location.DOM_KEY_LOCATION_STANDARD },
	Delete: { key: 'Delete', keycode: 46, location: location.DOM_KEY_LOCATION_STANDARD },
	PrintScreen: { key: 'PrintScreen', keycode: 46, location: location.DOM_KEY_LOCATION_STANDARD }, //?
	Pause: { key: 'Pause', keycode: 0, location: location.DOM_KEY_LOCATION_STANDARD }, //?

	// 受状态影响的 + Numpad0~Numpad9
	Digit0: { key: '0', shift: '!', keycode: 48, press: {keycode: 48, shift: 41}, location: location.DOM_KEY_LOCATION_STANDARD},
	Digit1: { key: '1', shift: '@', keycode: 49, press: {keycode: 49, shift: 33}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit2: { key: '2', shift: '#', keycode: 50, press: {keycode: 50, shift: 64}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit3: { key: '3', shift: '$', keycode: 51, press: {keycode: 51, shift: 35}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit4: { key: '4', shift: '%', keycode: 52, press: {keycode: 52, shift: 36}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit5: { key: '5', shift: '^', keycode: 53, press: {keycode: 53, shift: 37}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit6: { key: '6', shift: '&', keycode: 54, press: {keycode: 54, shift: 94}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit7: { key: '7', shift: '*', keycode: 55, press: {keycode: 55, shift: 38}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit8: { key: '8', shift: '(', keycode: 56, press: {keycode: 56, shift: 42}, location: location.DOM_KEY_LOCATION_STANDARD },
	Digit9: { key: '9', shift: ')', keycode: 57, press: {keycode: 57, shift: 40}, location: location.DOM_KEY_LOCATION_STANDARD },
	Backquote: { key: '`', shift: '~', keycode: 192, press: {keycode: 96, shift: 126}, location: location.DOM_KEY_LOCATION_STANDARD }, // `~
	Backslash: { key: '\\', shift: '|', keycode: 220, press: {keycode: 92, shift: 124}, location: location.DOM_KEY_LOCATION_STANDARD }, // \|
	BracketLeft: { key: '[', shift: '{', keycode: 219, press: {keycode: 91, shift: 123}, location: location.DOM_KEY_LOCATION_STANDARD }, // {[
	BracketRight: { key: ']', shift: '}', keycode: 221, press: {keycode: 93, shift: 125}, location: location.DOM_KEY_LOCATION_STANDARD },// }]
	Comma: { key: ',', shift: '<', keycode: 188, press: {keycode: 44, shift: 60}, location: location.DOM_KEY_LOCATION_STANDARD }, // ,<
	Equal: { key: '=', shift: '+', keycode: 187, press: {keycode: 61, shift: 43}, location: location.DOM_KEY_LOCATION_STANDARD }, // =+
	Minus: { key: '-', shift: '_', keycode: 189, press: {keycode: 45, shift: 95}, location: location.DOM_KEY_LOCATION_STANDARD }, // -_
	Period: { key: '.', shift: '>', keycode: 190, press: {keycode: 46, shift: 62}, location: location.DOM_KEY_LOCATION_STANDARD },// >.
	Quote: { key: '\'', shift: '"', keycode: 222, press: {keycode: 39, shift: 34}, location: location.DOM_KEY_LOCATION_STANDARD }, // '"
	Semicolon: { key: ';', shift: ':', keycode: 186, press: {keycode: 59, shift: 58}, location: location.DOM_KEY_LOCATION_STANDARD },// ;:
	Slash: { key: '/', shift: '?', keycode: 191, press: {keycode: 47, shift: 63}, location: location.DOM_KEY_LOCATION_STANDARD }, // /?
	NumpadAdd: { key: '+', keycode: 107, press: {keycode: 43}, location: location.DOM_KEY_LOCATION_NUMPAD },
	NumpadDecimal: { key: '.', keycode: 110, press: {keycode: 46}, location: location.DOM_KEY_LOCATION_NUMPAD },
	NumpadDivide: { key: '/', keycode: 191, press: {keycode: 47}, location: location.DOM_KEY_LOCATION_NUMPAD },
	NumpadEnter: { key: 'Enter', keycode: 13, press: {keycode: 13}, location: location.DOM_KEY_LOCATION_NUMPAD },
	NumpadMultiply: { key: '*', keycode: 106, press: {keycode: 42}, location: location.DOM_KEY_LOCATION_NUMPAD },
	NumpadSubtract: { key: '-', keycode: 109, press: {keycode: 45}, location: location.DOM_KEY_LOCATION_NUMPAD },

};

const FUNCTION_KEY_ORIGIN = 110;
(function loadFn() {
	for (let i = 1; i <= 12; i++) {
		keyboardMap[`F${i}`] = {
			key: `F${i}`,
			keycode: FUNCTION_KEY_ORIGIN + i,
			location: location.DOM_KEY_LOCATION_STANDARD
		};
	}
})();

const UPERCASE_LETTER_KEY_ORIGIN = 65;
const LOWERCASE_LETTER_KEY_ORIGIN = 97;
(function loadAlpha() {
	for (let i = 0; i < 26; i++) {
		keyboardMap['Key' + String.fromCharCode(UPERCASE_LETTER_KEY_ORIGIN + i)] = {
			key: String.fromCharCode(LOWERCASE_LETTER_KEY_ORIGIN + i),
			shift: String.fromCharCode(UPERCASE_LETTER_KEY_ORIGIN + i),
			keycode: UPERCASE_LETTER_KEY_ORIGIN + i,
			press:{
				keycode: LOWERCASE_LETTER_KEY_ORIGIN+i,
				shift: UPERCASE_LETTER_KEY_ORIGIN+i
			},
			location: location.DOM_KEY_LOCATION_STANDARD
		};
	}
})();

const NUMBER_KEY_ORIGIN = 48;
(function loadNum() {
	for (let i = 0; i < 10; i++) {
		keyboardMap[`Numpad${i}`] = {
			key: i,
			keycode: NUMBER_KEY_ORIGIN + i,
			press:{
				keycode: NUMBER_KEY_ORIGIN + i
			},
			location: location.DOM_KEY_LOCATION_NUMPAD
		};
	}
})();

exports.keyboardMap = keyboardMap;
exports.modifierMap = modifierMap;
exports.lockMap = lockMap;