const state = require('../../state');
const {createMouseEvent} = require('../../event');
const page = require('../../page');
const {Button} = require('../device');

state.register('mouse-button', {
	button: 0,
	buttons: 0o000
});

const DOUBLE_CLICK_DELAY = 500;
const BUTTON = {
	LEFT: {
		ID: 0,
		BIT: 0o001
	},
	MIDDLE: {
		ID: 1,
		BIT: 0o100
	},
	RIGHT: {
		ID: 2,
		BIT: 0o010
	}
};

class MouseButton extends Button {
	constructor(options, {ID, BIT}) {
		super(options);
		this.id = ID;
		this.bit = BIT;
	}
	down() {
		return super.down().then(() => {
			state.update('mouse-button', clone => {
				clone.buttons |= this.bit;
				clone.button = this.id;
			});

			page.dispatchEvent(createMouseEvent('mousedown'));
			page.setFocusElement();
		});
	}

	up() {
		return this.$checkReleased().then(() => {
			state.update('mouse-button', clone => {
				clone.buttons ^= this.bit;
				clone.button = this.id;
			});

			page.dispatchEvent(createMouseEvent('mouseup'));
			this.upFn();
		}, this.$stdOut);
	}
}

module.exports = {
	left: new MouseButton({
		down() {
			if (Date.now() - this.downTime < DOUBLE_CLICK_DELAY) {
				this.double = true;
			}
		},
		up() {
			if (state.get('mouse-button').button === this.id) {
				page.dispatchEvent(createMouseEvent('click'));
			}

			if (this.double) {
				this.double = false;
				page.dispatchEvent(createMouseEvent('dblclick'));
			}
		}
	}, BUTTON.LEFT),
	middle: new MouseButton({}, BUTTON.MIDDLE),
	right: new MouseButton({
		up() {
			page.dispatchEvent(createMouseEvent('contextmenu'));
		}
	}, BUTTON.RIGHT),
	release(buttonName = 'left') {
		return this[buttonName].up();
	},
	hold(buttonName = 'left') {
		return this[buttonName].down();
	},
	click(buttonName = 'left') {
		return Promise.resolve().then(() => {
			return this.hold(buttonName);
		}).then(() => {
			return this.release(buttonName);
		});
	},
	releaseAll() {
		['left', 'middle', 'right'].forEach(buttonName => {
			this.release(buttonName).catch(() => {});
		});
	}
};