const state = require('../../state');
const {animate} = require('../../util');
const {event, DeviceUnit} = require('../device');
const {createMouseEvent} = event;
const {updateHoverSheet} = require('../../page/hover');
const {
	fromPoint,
	queryCommonParent,
	queryParent
} = require('../../util').element;

state.register('pointer-position', {
	clientX: 0,
	clientY: 0
});

const viewport = {
	get height () {
		return window.innerHeight;
	},
	get width() {
		return window.innerWidth;
	},
};

function subRange(min, max, number) {
	if (number > max) return max;
	if (number < min) return min;
	return number;
}

function filterPoint(pos = state.get('pointer-position')) {
	pos.clientX = subRange(0, viewport.width, pos.clientX);
	pos.clientY = subRange(0, viewport.height, pos.clientY);
	return pos;
}

function enter(target, parent) {
	const dispatchingQueue = queryParent(target, parent);
	const mouseenterEvent = createMouseEvent('mouseenter');
	while (dispatchingQueue.length) {
		const element = dispatchingQueue.pop();
		element.dispatchEvent(mouseenterEvent);
		element.setAttribute('lc2-hover', true);
	}
}

function leave(target, parent) {
	const dispatchingQueue = queryParent(target, parent);
	const mouseleaveEvent = createMouseEvent('mouseleave');
	while (dispatchingQueue.length) {
		const element = dispatchingQueue.shift();
		element.dispatchEvent(mouseleaveEvent);
		element.removeAttribute('lc2-hover');
	}
}

const MOVE_LIMIT = 100;

module.exports = {
	moveTo(targetPoint, immediate = false) {
		const $position = state.get('pointer-position');
		const computedPoint = filterPoint(targetPoint);

		return new Promise(resolve => {
			const end = () => {
				this.pointTo(computedPoint);
				return resolve();
			};

			if (immediate) {
				state.set('pointer-position', computedPoint);
				end();
			} else {
				animate($position, computedPoint, {duration: MOVE_LIMIT})
					.onStart(() => this.pointTo($position))
					.onUpdate(() => this.pointTo($position))
					.onComplete(end)
					.start();
			}
		});
	},
	pointTo(position) {
		state.emit('point-query-start', position);

		// Dispatch mousemove.
		const target = fromPoint(position.clientX, position.clientY);
		const mousemoveEvent = createMouseEvent('mousemove');
		target.dispatchEvent(mousemoveEvent);

		// Update page currentElement.
		// Dispatch mouseenter & mouseleave.
		let lastTarget = state.get('page').currentElement;
		if (lastTarget !== target) {
			updateHoverSheet();
			const commonParent = queryCommonParent(lastTarget, target);
			
			//TODO 对内部元素生效 mouseout when mousedown left
			lastTarget.dispatchEvent(createMouseEvent('mouseout'));
			leave(lastTarget, commonParent);

			//TODO 对内部元素生效 mouseover when mousedown left
			target.dispatchEvent(createMouseEvent('mouseover'));
			enter(target, commonParent);
			state.update('page', clone => {
				clone.currentElement = target;
			});
		}

		state.emit('point-query-end', position);
		
		state.set('pointer-position', position);
	}
};