let isECS = true, hasInput = true;
try { new MouseEvent('click', {}); } catch (_) { isECS = false; }
try { InputEvent; } catch (_) { hasInput = false; }

function buildInitEventArgs(type, opts, map) {
	const args = [type];
	for (let optKey in map) {
		if (map.hasOwnProperty(optKey)) {
			args[map[optKey]] = opts[optKey] || null;
		}
	}
	return args;
}

const $Event = isECS ? Event : function Event (typeArg, eventInit) {
	const event = document.createEvent('Event'),
		initMap = { 'bubbles': 1, 'cancelable': 2};
	event.initEvent.apply(event, buildInitEventArgs(typeArg, eventInit, initMap));
	return event;
};

const $MouseEvent = isECS ? MouseEvent : function MouseEvent (typeArg, eventInit) {
	var event = document.createEvent('MouseEvent'),
		initMap = {
			'bubbles': 1,
			'cancelable': 2,
			'view': 3,
			'detail': 4,
			'screenX': 5,
			'screenY': 6,
			'clientX': 7,
			'clientY': 8,
			'ctrlKey': 9,
			'altKey': 10,
			'shiftKey': 11,
			'metaKey': 12,
			'button': 13,
			'relatedTarget': 14
		};
	event.initMouseEvent.apply(event, buildInitEventArgs(typeArg, eventInit, initMap));
	return event;
};

const $UIEvent = isECS ? UIEvent : function (typeArg, eventInit) {
	var event = document.createEvent('UIEvent'),
		initMap = { 'bubbles': 1, 'cancelable': 2, 'view': 3, 'detail': 4 };
	event.initUIEvent.apply(event, buildInitEventArgs(typeArg, eventInit || {}, initMap));
	return event;
};

const $KeyboardEvent = isECS ? KeyboardEvent : function (typeArg, eventInit, readOnly) {
	
	// TODO modifiersList, locale的特殊处理
	var event = $UIEvent(typeArg, eventInit),
		// initMap = { 'bubbles': 1, 'cancelable': 2, 'view': 3,
		// 	'char': 4, key: 5, location: 6, repeat: 8 };
		initMap = ['key', 'location', 'modifiersList', 'repeat', 'locale'];
	
	for(let key of initMap) {
		event[key] = eventInit[key];
	}

	for(let key in readOnly) {
		event[key] = readOnly[key];
	}
	
	return event;
};

const $InputEvent = hasInput ? InputEvent : $UIEvent;

const $FocusEvent = isECS ? FocusEvent : function (typeArg, eventInit) {
	var event = document.createEvent('FocusEvent'),
		initMap = { 'bubbles': 1, 'cancelable': 2, 'view': 3, 'detail': 4 };
	event.initUIEvent.apply(event, buildInitEventArgs(typeArg, eventInit || {}, initMap));
	return event;
};

module.exports = {
	MouseEvent: $MouseEvent,
	UIEvent: $UIEvent,
	KeyboardEvent: $KeyboardEvent,
	FocusEvent: $FocusEvent,
	Event: $Event
};