const state = require('../state');
const {
	MouseEvent,
	UIEvent,
	KeyboardEvent,
	FocusEvent,
	Event
} = require('./constructor');

function markEvent(event) {
	return Object.assign(event, {__LC__: true});
}

const viewport = {
	get top() {
		return window.screenTop + window.outerHeight - window.innerHeight;
	},
	get left() {
		return window.screenLeft;
	}
};

function computeScreenByClient(client) {
	const screen = {
		screenX: client.clientX + viewport.left,
		screenY: client.clientY + viewport.top
	};

	return Object.assign({}, client, screen);
}

const validMouseEvent = [
	'mousedown', 'mouseup', 'mouseover', 'mouseout',
	'mouseenter', 'mouseleave', 'click', 'dblclick', 'mousemove',
	'contextmenu'
];
const noBubbleMouseEvent = ['mouseenter', 'mouseleave'];
const validKeyboardEvent = ['keydown', 'keyup', 'keypress'];

const defaultOptions = {bubbles: true, cancelable: true};

/**
 * @param {Object} options options to event constructor.
 */
exports.createMouseEvent = function (type) {
	if (validMouseEvent.indexOf(type) === -1) {
		throw new Error(`Invalid mouse event type: ${type}`);
	}

	let bubbles = noBubbleMouseEvent.indexOf(type) === -1;

	const position = computeScreenByClient(state.get('pointer-position'));
	const button = state.get('mouse-button');
	const modifier = state.get('modifier');

	const mouseEventInitOptions =
		Object.assign({}, defaultOptions, {bubbles}, position, button, modifier);
	const event = new MouseEvent(type, mouseEventInitOptions);

	return markEvent(event);
};

exports.createKeyboardEvent = function (type, {args, readOnly}) {
	if (validKeyboardEvent.indexOf(type) === -1) {
		throw new Error(`Invalid keyboard event type: ${type}`);
	}

	const modifier = state.get('modifier');

	const keyboardEventInitOptions = Object.assign({}, defaultOptions, args, modifier);
	const event = new KeyboardEvent(type, keyboardEventInitOptions, readOnly);

	for(let key in readOnly) {
		delete event[key];
		Object.defineProperty(event, key, {value: readOnly[key]}); 
	}

	return markEvent(event);
};

exports.createEvent = function (type, option = defaultOptions) {
	return new Event(type, option);
};

exports.createTouchEvent = function () {

};

exports.createWheelEvent = function () {

};

exports.createUIEvent = function (type) {
	return markEvent(new UIEvent(type, {bubbles: false, cancelable: true}));
};

exports.createFocusEvent = function (type) {
	return markEvent(new FocusEvent(type, defaultOptions));
};