const {mouse} = require('../device');
const {getElementClientPosition} = require('../util/element');
const {createEvent} = require('../event');
class Form {
	constructor (form) {
		this.form = form;
		this.submitButton = this.getSubmitButton(this.form);
	}

	getSubmitButton(root) {
		const children = root.children;
		for( let i = 0 ; i < children.length ; i++ ) {
			const element = children[i];
			
			if((element.tagName === 'INPUT' && element.type === 'submit') ||
				(element.tagName === 'BUTTON' && element.type === 'submit')) {
				return element;
			}
			const submitButton = this.getSubmitButton(element);
			if(submitButton) return submitButton;
		}
		return null;
	}

	submit() {
		if(this.submitButton) {
			return mouse.moveTo(getElementClientPosition(this.submitButton)).then(() => {
				return mouse.clickButton();
			});
		}
		if(this.form.getElementsByTagName('input').length !== 1 ||
			(this.form.onsubmit && !this.form.onsubmit())) {
			return Promise.resolve();
		}

		this.form.dispatchEvent(createEvent('submit', {bubbles: false, cancelable: true}));
		return Promise.resolve();
	}
}
exports.formToSubmit = function (element) {
	if(!(element && element.tagName === 'INPUT')) return null;
	const form = getFormNode(element);
	return form && new Form(form);
};

const getFormNode = exports.getFormNode = function (element) {
	let currentElement = element;
	while(currentElement && currentElement.tagName !== 'BODY') {
		if(currentElement.tagName === 'FORM') return currentElement;
		currentElement = currentElement.parentNode;
	}
	return null;
};

