'use strict';
const {traverseFrame} = require('../util');
const hoverReg = /:hover/g;

function searchHoverRule(document) {
	const ruleList = [];
	const styleSheetList = document.styleSheets;

	for (let styleSheet of Array.from(styleSheetList)) {
		for(let rule of Array.from(styleSheet.cssRules)) {
			if (rule.selectorText && rule.selectorText.match(hoverReg)) {
				ruleList.push(rule);
			}
		}
	}

	return ruleList;
}

function parseHoverRule(ruleList) {
	let cssText = '';

	for(let rule of ruleList) {
		cssText += rule.cssText.replace(hoverReg, '[lc2-hover]');
	}

	return cssText;
}
function updateHoverSheet() {
	traverseFrame(window, window => {
		const lastStyle = window.__LC2_HOVER_STYLE__;
		if (lastStyle && lastStyle.parentNode) {
			lastStyle.parentNode.removeChild(lastStyle);
		}
		
		const styleElement =
			window.__LC2_HOVER_STYLE__ =
			window.document.createElement('style');

		const bodyElement = window.document.body;
		bodyElement && bodyElement.appendChild(styleElement);

		const hoverRuleList = searchHoverRule(window.document);
		const hoverCssText = parseHoverRule(hoverRuleList);
		styleElement.innerText = hoverCssText;
	});
}
exports.updateHoverSheet = updateHoverSheet;