/**
 * Register some invoking before frame ready
 */
const {traverseFrame} = require('../util');
const state = require('../state');

state.register('prevent-signal', {
	prevent: true,
	uploading: false,
	target: null
});

state.register('page-native', {
	alert: window.alert,
	confirm: window.confirm,
	prompt: window.prompt,
});

function setSignalBlocked(target) {
	state.update('prevent-signal', clone => {
		clone.target = target;
		clone.uploading = true;
	});
}

function cancelSignalBlocked() {
	state.update('prevent-signal', clone => {
		clone.target = null;
		clone.uploading = false;
	});
}

let alertLastMessage = '';
let confirmLastMessage = '';
let confirmReturn = true;
let promptLastMessage = '';
let promptValue = '';

setInterval(() => traverseFrame(window, window => {
	// Check this window of frame has been injected or not.
	if (window.lc2Injected) {
		return;
	}

	// Prevent upload request.
	window.addEventListener('click', (event) => {
		const {target} = event;
		if (target && state.get('prevent-signal').prevent && target.type === 'file') {
			state.emit('upload-request', target);
			event.preventDefault();
			setSignalBlocked(target);
		}
	});

	// Prevent upload end
	window.addEventListener('change', ({target}) => {
		if (target && state.get('prevent-signal').prevent && target.type === 'file') {
			state.emit('upload-end');
			cancelSignalBlocked();
		}
	});

	window.lc2Injected = true;

	window.alert = function (message) {
		if (!state.get('prevent-signal').prevent) {
			return state.get('page-native').alert.call(null, message);
		}

		// 抽象
		state.emit('page-alert', message);
		alertLastMessage = message;

		return undefined;
	};

	window.confirm = function (message) {
		if (!state.get('prevent-signal').prevent) {
			return state.get('page-native').confirm.call(null, message);
		}

		state.emit('page-confirm', message);
		confirmLastMessage = message;

		return confirmReturn;
	};
	
	window.prompt = function (message) {
		if (!state.get('prevent-signal').prevent) {
			return state.get('page-native').prompt.call(null, message);
		}

		state.emit('page-prompt', message);
		promptLastMessage = message;

		return promptValue;
	};
}), 50);

exports.setConfirmReturn = function (value) {
	confirmReturn = Boolean(value);
};

exports.setPromptValue = function (value) {
	promptValue = value + '';
};

exports.checkAlertMessage = function (message) {

};

exports.checkConfirmMessage = function (message) {

};

exports.checkPromptMessage = function (message) {

};

exports.setSignalBlocked = setSignalBlocked;
exports.cancelSignalBlocked = cancelSignalBlocked;