const EventEmitter = require('events');

const storage = {};
const state = module.exports = new EventEmitter();

function $getGroup(groupName) {
	const group = storage[groupName];
	if (!group) {
		throw new Error(`The group(${groupName}) is not defined`);
	}

	return group;
}

state.register = function (groupName, originState = {}) {
	if (storage[groupName]) {
		throw new Error(`Group name(${groupName}) is defined.`);
	}

	return storage[groupName] = Object.assign({}, originState);
};

state.set = function (groupName, newState = {}, silence = false) {
	const group = $getGroup(groupName);

	for(let key in group) {
		const newValue = newState[key];
		if (newValue !== undefined) {
			group[key] = newValue;
		}
	}

	if (!silence) {
		this.emit(`${groupName}-change`, this.get(groupName));
	}
};

state.get = function (groupName) {
	return Object.assign({}, $getGroup(groupName));
};

state.update = function (groupName, callback) {
	const stateClone = this.get(groupName);
	callback(stateClone);
	this.set(groupName, stateClone);
};

state.error = function (title, err) {
	this.emit('error', {title, err});
};