class KeyAnimation {
	/**
 * @param {string} key
 * @param {object} modifier
 */
	constructor(key, modifier) {
		this.key = key;
		this.ctrlkey = Boolean(modifier.ctrlkey);
		this.shiftkey = Boolean(modifier.shiftkey);
		this.altkey = Boolean(modifier.altkey);
		this.metakey = Boolean(modifier.metakey);
		/** @type {Document} */
		this.doc = modifier.doc || document;
	}

	start() {
		const div = this.doc.createElement('div');
		this.config(div);
		const svg = '<svg><text>' + this.key + '</text></svg>';
		div.innerHTML = svg;
		this.doc.appendChild(div);

		setTimeout(() => {
			div.remove();
		}, 1000);

		return div;
	}

	config(div) {
		div.class = 'lc-driver-key-animation';
		div.style.position = 'fixed';
		div.style.zIndex = 999999;
		div.style.width = 64;
		div.style.height = 64;
		div.style.backgroundColor = 'transparent';
	}
}

/**
 * @param {string} key - a key to print on screen
 * @param {object=} modifier - options
 * @return {HTMLDivElement}
 */
module.exports = function showKey(key, modifier = {}) {
	if (typeof key !== 'string') {
		throw new Error('expect a key');
	} 
	return new KeyAnimation(key, modifier).start();
};
