'use strict';
/**
 * A transition class for linear animation.
 * @class
 */
class Transition {
	/**
	 * @param {Object} from
	 * @param {Object} to
	 * @param {Number} [duration] Time for transition to complete in milliseconds.
	 * @param {Number} [rate] Update frequency in Hz.
	 */
	constructor(from, to, {duration = 100, rate = 100}) {
		this.$from = from;
		this.$to = to;
		this.$duration = duration;
		this.$rate = rate;
		this.$onStart = Transition.empty;
		this.$onUpdate = Transition.empty;
		this.$onComplete = Transition.empty;
	}

	onStart(start = Transition.empty){
		this.$onStart = start;
		return this;
	}

	onUpdate(update = Transition.empty) {
		this.$onUpdate = update;
		return this;
	}

	onComplete(complete = Transition.empty) {
		this.$onComplete = complete;
		return this;
	}

	start() {
		// Calculate total update counts.
		const speed = this.$duration*this.$rate/1000.0;
		const step = {};
		for(let o in this.$to) {
			const delta = this.$to[o]-this.$from[o];
			if(typeof delta === 'number' && delta) {
				step[o] = delta/speed;
			}
		}

		this.$onStart();

		if(!Object.getOwnPropertyNames(step).length) {
			this.$onComplete();
			return;
		}

		var interval = setInterval(() => {
			for( let o in step) {
				this.$from[o] += step[o];
				// Check if transition completed.
				if((step[o] < 0 && this.$from[o] <= this.$to[o]) || (
					step[o] > 0 && this.$from[o] >= this.$to[o])) {
					clearInterval(interval);
					for(let o in step) {
						this.$from[o] = this.$to[o];
					}
					this.$onUpdate();
					this.$onComplete();
					return;
				}
			}
			this.$onUpdate();

		}, 1000.0/this.$rate);
	}

	static empty() {}
}

module.exports = function animate(from, to, options = {}) {
	if(!(from && to)) {
		throw new Error('Invalid parameter: from & to must be an object.');
	}
	return new Transition(from, to, options);
};