const animate = require('../src/util/animate');
const assert = require('assert');

describe('animate::', function () {
	it('with default option', function (done) {
		const start_pos = {
			clientX: 50,
			clientY: -20
		};
		const end_pos = {
			clientX: -10,
			clientY: 100
		};
		animate(start_pos,end_pos)
		.onComplete(() => {
			assert.equal(start_pos.clientX, end_pos.clientX);
			assert.equal(start_pos.clientY, end_pos.clientY);
			done();
		}).start();
	});

	it('with set duration and update rate', function (done) {
		const start_pos = {
			clientX: -10,
			clientY: -20
		};
		const end_pos = {
			clientY: 100
		};
		animate(start_pos, end_pos, {duration: 1000, rate: 10})
		.onComplete(() => {
			assert.equal(start_pos.clientX, -10);
			assert.equal(start_pos.clientY, end_pos.clientY);
			done();
		}).start();
	});

	it('throw error', function () {
		assert.throws(() => {
			animate().start();
		}, 'Start and end status must be set.');
	});

	it('onStart', function (done) {
		var start = false;
		const from = {x: 0};
		const to = {x:100};
		animate(from, to).onStart(() => {
			start = true;
		}).onComplete(() => {
			assert.equal(from.x, to.x);
			assert.equal(start, true);
			done();
		}).start();
	});

	it('onUpdate', function (done) {
		var updateCnt = 0;
		const from = {x: 0};
		const to = {x:100};
		animate(from, to, {duration: 100, rate:100}).onUpdate(() => {
			updateCnt++;
		}).onComplete(() => {
			assert.equal(updateCnt, 10);
			done();
		}).start();
	});

	it('start and end with same pos', function (done) {
		const start_pos = {
			clientX: 50,
			clientY: -20
		};
		const end_pos = {
			clientX: 50,
			clientY: -20
		};
		animate(start_pos,end_pos)
		.onComplete(() => {
			assert.equal(start_pos.clientX, end_pos.clientX);
			assert.equal(start_pos.clientY, end_pos.clientY);
			done();
		}).start();
	});

	it('with one property equal', function (done) {
		var updateCnt = 0;
		const start_pos = {
			clientX: 50,
			clientY: -20
		};
		const end_pos = {
			clientX: 50,
			clientY: -100
		};
		animate(start_pos,end_pos)
		.onUpdate(() => {
			updateCnt++;
		})
		.onComplete(() => {
			assert.equal(updateCnt, 10);
			assert.equal(start_pos.clientX, end_pos.clientX);
			assert.equal(start_pos.clientY, end_pos.clientY);
			done();
		}).start();
	});
});