const {Button} = require('../src/device/device');
const assert = require('assert');

describe.only('Component::', function () {
	describe('Button::', function () {
		it('constructor', function () {
			const button = new Button({});

			assert.equal(button.holding, false);
			assert.equal(button.downTime, 0);
			assert.equal(button.downFn, Button.noop);
			assert.equal(button.upFn, Button.noop);
		});

		it('down', function (done) {
			var signal = false;
			const downFn = () => {
				signal = true;
			};
			const button = new Button({down: downFn});

			button.down().then(() => {
				assert.equal(button.holding, true);
				assert.notEqual(button.downTime, 0);
				assert.equal(signal, true);
				done();
			});
		});

		it('down twice', function (done) {
			var signal = false;
			const upFn = () => {
				signal = true;
			};
			const button = new Button({up: upFn});

			button.down().then(() => {
				return button.down();
			}).then(() => {
				assert.equal(button.holding, true);
				assert.equal(signal, true);
				done();
			});
		});

		it('down then up', function (done) {
			var downSignal = false, upSignal = false;
			const upFn = () => {
				upSignal = true;
			};
			const downFn = () => {
				downSignal = true;
			};
			const button = new Button({down: downFn, up: upFn});

			button.down().then(() => {
				return button.up();
			}).then(() => {
				assert.equal(button.holding, false);
				assert.equal(upSignal, true);
				assert.equal(downSignal, true);
				done();
			});
		});

		it('up without down', function (done) {
			const button = new Button({});
			button.up().catch(err => {
				assert.equal(err.message, 'The button has been released.');
				done();
			});
		});
	});
});